from pathlib import Path
import yaml


boot_config_path = Path('/boot/wildflower-config.yml')
if boot_config_path.exists():
    with boot_config_path.open('r', encoding="utf-8") as fp:
        config = yaml.safe_load(fp.read())
else:
    config = {
        "device_id": "deadbeef",
        "assignment-id": "deadbeef",
        "environment-id": "deadbeef",
    }

DEVICE_ID = config.get("device_id", "unknown")
ASSIGNMENT_ID = config.get("assignment-id", "unknown")
ENV_ID = config.get("environment-id", "unknown")



import functools
import io
import logging
import os
from pathlib import Path

from minio import Minio
from minio.error import MinioException
import yaml

from .config import DEVICE_ID

BUCKET = os.environ.get("MINIO_BUCKET_NAME", "videos")


@functools.cache
def get_client():
    minioClient = Minio(os.environ.get("MINIO_URL", "minio-service.classroom.svc.cluster.local:9000"),
                        access_key=os.environ.get("MINIO_KEY", "wildflower-classroom"),
                        secret_key=os.environ.get("MINIO_SECRET"),
                        secure=False)
    try:
        minioClient.make_bucket(BUCKET, location="us-east-1")
    except MinioException:
        pass
    return minioClient


def send_to_minio(data_dir, video_metadata):
    name = video_metadata["name"]
    rel_path = video_metadata["meta"]["path"]
    ts = video_metadata["timestamp"]
    try:
        obj_name = (f'{DEVICE_ID}/{rel_path}.meta')
        logging.info("putting %s metadata on minio", obj_name)
        data = yaml.safe_dump(video_metadata)
        get_client().put_object(BUCKET, obj_name, io.BytesIO(data.encode("utf8")), len(data), content_type='text/yaml', metadata={
                                "source": DEVICE_ID,
                                "ts": ts,
                                })
        obj_name = (f'{DEVICE_ID}/{rel_path}')
        logging.info("putting %s on minio", obj_name)
        get_client().fput_object(BUCKET, obj_name, data_dir / name, content_type='video/mp4', metadata={
                                "source": DEVICE_ID,
                                "ts": ts,
                                })
        # Path(data_dir / name).rename(data_dir / f"_{name}_")
        Path(data_dir / name).unlink()
    except MinioException as err:
        logging.info(err)
        logging.info("failed to process %s", name)



import datetime
import logging
from multiprocessing import Process, Queue
import os
import time

import yaml
from minio import Minio
from minio.error import MinioException
from picamera2 import Picamera2
from picamera2.encoders import Quality
from picamera2.encoders import H264Encoder
from picamera2.outputs import FfmpegOutput
from libcamera import Transform


with open('/boot/wildflower-config.yml', 'r', encoding="utf-8") as fp:
    config = yaml.safe_load(fp.read())


DEVICE_ID = config.get("device_id", "unknown")
BUCKET = os.environ.get("MINIO_BUCKET_NAME", "videos")

CAMERA_RES = (os.environ.get("CAMERA_RES_W", 1296), os.environ.get("CAMERA_RES_H", 972))
CAMERA_FRAMERATE = os.environ.get("CAMERA_FRAMERATE", 10)
INTRA_PERIOD = os.environ.get("INTRA_PERIOD", 120)
BITRATE = os.environ.get("BITRATE", 1000000)
PERIOD = os.environ.get("DURATION", 120)
# CAMERA_ISO_SETTING = int(os.environ.get("CAMERA_ISO_SETTING", 400))
# CAMERA_EXPOSURE_MODE = os.environ.get("CAMERA_EXPOSURE_MODE", 'sports')
# CAMERA_SHUTTER_SPEED = int(os.environ.get("CAMERA_SHUTTER_SPEED", 0))
CAMERA_H_FLIP = os.environ.get("CAMERA_H_FLIP", 0)
CAMERA_V_FLIP = os.environ.get("CAMERA_V_FLIP", 0)




def next_timeslot(now):
    return now + (PERIOD - (now % PERIOD))


def capture_loop():
    logging.info("starting capture")
    try:
        transform = Transform()
        if CAMERA_H_FLIP=="yes" and CAMERA_V_FLIP=="yes":
            transform = Transform(hflip=True, vflip=True)
        elif CAMERA_H_FLIP=="yes":
            transform = Transform(hflip=True, vflip=False)
        elif CAMERA_V_FLIP=="yes":
            transform = Transform(hflip=False, vflip=True)
        camera = Picamera2()
        with camera:
            video_config = camera.create_video_configuration(main={"size": CAMERA_RES}, transform=transform, controls={"FrameRate": CAMERA_FRAMERATE})
            camera.configure(video_config)

            encoder = H264Encoder(BITRATE)

            logging.info(f"period is {PERIOD}")

            now = time.time()
            timeslot = next_timeslot(now)
            sleep_time = timeslot - now

            if sleep_time < 2:  # Ensure camera has enough time to adjust
                sleep_time += PERIOD
                timeslot += PERIOD
            logging.info(f"going to sleep for a bit {sleep_time}")
            time.sleep(sleep_time)

            video_start_time = datetime.datetime.fromtimestamp(timeslot)
            name = f'/data/video-{video_start_time:%Y_%m_%d_%H_%M-%S}.mp4'
            output = FfmpegOutput(name)
            while True:
                camera.start_and_record_video(output, encoder=encoder, quality=Quality.VERY_HIGH, duration=PERIOD)
                # camera.stop_recording()
                timeslot += PERIOD
                video_start_time = datetime.datetime.fromtimestamp(timeslot)
                name = f'/data/video-{video_start_time:%Y_%m_%d_%H_%M-%S}.mp4'
                output = FfmpegOutput(name)
                # camera.start_recording(encoder, output, quality=Quality.VERY_HIGH)
                # delay = timeslot + PERIOD - time.time()
                # logging.info(f"waiting {delay}")
                # time.sleep(delay)
    except Exception as e:
        logging.info(e)



def get_next_file():
    for item in os.listdir('/data'):
        if item.endswith(".mp4"):
            fname = f'/data/{item}'
            st = os.stat(fname)
            if (time.time() - st.st_mtime) > 11 and st.st_size > 300000:
                return fname
    return None


def upload_loop():
    minioClient = Minio(os.environ.get("MINIO_URL", "minio-service.classroom.svc.cluster.local:9000"),
                        access_key=os.environ.get("MINIO_KEY", "wildflower-classroom"),
                        secret_key=os.environ.get("MINIO_SECRET"),
                        secure=False)
    try:
        minioClient.make_bucket(BUCKET, location="us-east-1")
    except MinioException:
        pass
    while True:
        name = get_next_file()
        if name:
            try:
                ts = name[12:-4]
                obj_name = (f'{DEVICE_ID}/{ts}.mp4').replace("_", "/")
                logging.info(f"putting {obj_name} on minio")
                minioClient.fput_object(BUCKET, obj_name, name, content_type='video/mp4', metadata={
                                        "source": DEVICE_ID,
                                        "ts": ts,
                                        "duration": f"{PERIOD}s",
                                        })
                os.remove(name)
            except MinioException as err:
                logging.info(err)
                logging.info(f"failed to process {name}")
        else:
            logging.info("no files")
            time.sleep(1)



import logging
from pathlib import Path

# from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

from capture.segmentdata import SegmentDataIndex


class ChangeHandler(FileSystemEventHandler):

    def on_modified(self, event):
        pass


def monitor_directory(str_path):
    path = Path(str_path)
    logging.info(path)
    index = SegmentDataIndex(path)

    files = sorted(path.glob("*.h264"))
    for file in files:
        index.add_video(file.relative_to(path))
    # event_handler = ChangeHandler()
    # observer = Observer()
    # observer.schedule(event_handler, path, recursive=True)
    # observer.start()
    # try:
    #     while True:
    #         time.sleep(1)
    # finally:
    #     observer.stop()
    #     observer.join()



from dataclasses import dataclass
import logging
import math
import sqlite3

import pendulum
import yaml

from .config import DEVICE_ID, ENV_ID, ASSIGNMENT_ID
from .vidtools import get_video_file_details
from .sokobon import send_to_minio



@dataclass(repr=False)
class Timecode:
    hour: int = 0
    minute: int = 0
    second: int = 0
    frame: int = 0

    def __repr__(self):
        return f"{self.hour:02}:{self.minute:02}:{self.second:02}.{self.frame:02}"

    @classmethod
    def from_string(cls, value: str) -> "Timecode":
        clock, frame = value.split(".")
        hour, minute, second = clock.split(":")
        return Timecode(hour=hour, minute=minute, second=second, frame=frame)

    @classmethod
    def from_offset(cls, value: float, fps: float) -> "Timecode":
        ms = int(value)
        second = int((ms / 1000) % 60)
        minute = int((ms / 1000 / 60) % 60)
        hour = int((ms / 1000 / 60 / 60)  % 60)
        frame = math.ceil((ms % 1000) / (1000/fps))
        if frame == fps:
            frame = 0
            second += 1
        return Timecode(hour=hour, minute=minute, second=second, frame=frame)


@dataclass
class VideoInfo:
    id: int
    name: str
    offset: float = 0.0
    uploaded: bool = False
    frame_count: int = 0
    timecode: Timecode = Timecode()
    fps: int = 0
    duration: float = 0.0


class InvalidDirectory(Exception):

    def __init__(self, path):
        self.path = path
        super().__init__(f"specified [{path}] path does not contain required `wf-video-index.yml` or `start.timestamp` files")


class MissingZeroVideo(Exception):

    def __init__(self, path):
        self.path = path
        super().__init__(f"specified [{path}] path does not contain the first video in the sequence")


class SegmentDataIndex():

    def __init__(self, path):
        self.path = path
        # timestamp calculated after reading all other times
        self.start_datetime = None
        # timestamp written to the `start.timestamp` file
        self.start_approx_ts = None
        # create time from stat for the `start.timestamp` file
        self.start_ts_create = None
        # create time from stat for the first video
        self.start_zero_create = None
        logging.info("SegmentDataIndex startup %s", path)
        self.dbcon = None
        self._init_index()
        self._load_timestamps()

    def _init_index(self):
        index_file_path = self.path / "wf-video-index.yml"
        start_ts_file = self.path / "start.timestamp"
        if index_file_path.exists():
            logging.info("indexing already started, resuming")
            with index_file_path.open() as fp:
                index_info = yaml.safe_load(fp.read())
            self.start_datetime = pendulum.parse(index_info.get("start_datetime"))
            self.start_approx_ts = pendulum.parse(index_info.get("start_approx_ts"))
            self.start_ts_create = pendulum.parse(index_info.get("start_ts_create"))
            self.start_zero_create = pendulum.parse(index_info.get("start_zero_create"))
        elif start_ts_file.exists():
            self._perform_start_calculation()
        else:
            raise InvalidDirectory(self.path)
        self.dbcon = sqlite3.connect(self.path / "index.db")
        cur = self.dbcon.cursor()
        cur.execute('CREATE TABLE IF NOT EXISTS main.videos (id int PRIMARY KEY, name text, offset float, uploaded bool, frame_count int, fps int, timecode text, duration float)')
        cur.execute('CREATE TABLE IF NOT EXISTS main.frames (id int PRIMARY KEY, offset float)')
        self.dbcon.commit()
        cur.close()

    def _perform_start_calculation(self):
        """
        This is a sort of messy way to calculate the actual start time of the
        video capture. When capture starts a timestamp is written to "start.timestamp"
        Then capture kicks off. The timestamp of the first video file is the preferred
        timestamp to use, as it should align with when actual capture started.
        If that timestamp is somehow in the past of more than 10 seconds later
        then we reject it. We next look at the create time for the "start.timestamp"
        file. If that is in the past or more than 10 seconds later then we assme the
        timestamp in the file is the best candidate. We add a millesecond to that
        to account for startup time for libcamera, which may be naive.
        """
        start_ts_file = self.path / "start.timestamp"
        zero_video =  self.path / "video-00000.h264"
        if not zero_video.exists():
            raise MissingZeroVideo(self.path)
        logging.info("start timestamp found")
        with start_ts_file.open() as fp:
            ts_str = fp.read()
            logging.info(ts_str.strip())
        self.start_approx_ts = pendulum.parse(f'{ts_str[:-4]}+0000')
        self.start_ts_create = pendulum.from_timestamp(start_ts_file.stat().st_birthtime, 'UTC')
        self.start_zero_create = pendulum.from_timestamp(zero_video.stat().st_birthtime, 'UTC')
        aprox_create_diff = (self.start_ts_create - self.start_approx_ts).total_seconds()
        aprox_zero_diff = (self.start_zero_create - self.start_approx_ts).total_seconds()
        if (0 > aprox_create_diff > 10) and (0 > aprox_zero_diff > 10):
            # files probably moved or some sort of pause happened.
             # naive assumption that it took 1ms for capture to start after timestamp was written
            self.start_datetime = self.start_approx_ts.add(seconds=0.0001)
        elif 0 < aprox_zero_diff < 10:
            # prefer the zero create time over others.
            self.start_datetime = self.start_zero_create
        else:
            # fallback to create time of the ts file.
            self.start_datetime = self.start_zero_create
        self._write_index()

    def _write_index(self):
        index_file_path = self.path / "wf-video-index.yml"
        with index_file_path.open('w') as fp:
            fp.write(yaml.safe_dump({
                "start_datetime": self.start_datetime.isoformat(),
                "start_approx_ts": self.start_approx_ts.isoformat(),
                "start_ts_create": self.start_ts_create.isoformat(),
                "start_zero_create": self.start_zero_create.isoformat(),
            }))
            fp.flush()

    def find_video_for_index(self, index):
        cur = self.dbcon.cursor()
        existing = cur.execute("SELECT * FROM main.videos WHERE id = ?", [index]).fetchall()
        if len(existing) < 1:
            return None
        existing = existing[0]
        return VideoInfo(
            id=existing[0],
            name=existing[1],
            offset=existing[2],
            uploaded=existing[3],
            frame_count=existing[4],
            fps=existing[5],
            timecode=Timecode.from_string(existing[6]),
            duration=existing[6],
        )

    def add_video(self, path):
        cur = self.dbcon.cursor()
        # do we already have and existing entry, if so check that it is correct.
        index = int(path.name[6:-5], 10)
        existing = self.find_video_for_index(index)
        if existing:
            logging.info("video exists in the index [%s]", path.name)
        else:
            logging.info("new video added to index [%s]", path.name)
            cur.execute('INSERT OR IGNORE INTO main.videos (id, name, offset, uploaded, frame_count, fps, timecode) VALUES (?, ?, 0, false, 0, 0, ?)', [index, path.name, "00:00:00.00"])
            existing = self.find_video_for_index(index)
        is_dirty = False
        if index == 0 and existing.offset != 0:
            existing.offset = 0
            existing.timecode = Timecode()
            is_dirty = True
        if existing.frame_count == 0 or existing.fps == 0:
            video_properties = get_video_file_details(self.path / path)
            existing.fps = video_properties.get("fps")
            existing.duration = video_properties.get("duration")
            existing.frame_count = video_properties.get("frame_count")
        if index and existing.offset <= 0.0:
            previous = self.find_video_for_index(index-1)
            if previous:
                offset = previous.offset
                frame = cur.execute("SELECT id FROM main.frames WHERE offset = ?", [offset]).fetchone()
                if frame:
                    previous_frame_index = int(frame[0])
                    frame = cur.execute("SELECT offset FROM main.frames WHERE id = ?", [existing.frame_count + previous_frame_index]).fetchone()
                    if frame:
                        existing.offset = frame[0]
                        existing.timecode = Timecode.from_offset(float(frame[0]), existing.fps)
                        is_dirty = True
        if is_dirty:
            logging.info("saving updated meta into to index [%s]", path.name)
            cur.execute('UPDATE main.videos SET id = ?, name = ?, offset = ?, uploaded = ?, frame_count = ?, fps = ?, timecode = ?, duration = ? WHERE id = ?', [
                existing.id,
                existing.name,
                existing.offset,
                existing.uploaded,
                existing.frame_count,
                existing.fps,
                str(existing.timecode),
                existing.duration,
                existing.id,
            ])
        if not existing.uploaded and (index == 0 or existing.offset >= 0.0) and existing.frame_count > 0:
            logging.info("sending %s to be uploaded now", path.name)
            logging.info("ok, offset is %s", existing.offset)
            send_to_minio(self.path, {
                "name": path.name,
                "timestamp": self.timestamp_for_offset(existing.offset).isoformat(),
                "meta": {
                    "environment_id": ENV_ID,
                    "assignment_id": ASSIGNMENT_ID,
                    "camera_id": DEVICE_ID,
                    "path": str((self.path / path).relative_to(self.path.parent)),
                    "duration_seconds": existing.duration,
                    "fps": existing.fps,
                    "frame_offsets": self.load_offsets(existing.offset, existing.frame_count),
                }
            })
            cur.execute('UPDATE main.videos SET uploaded = true WHERE id = ?', [existing.id])
            logging.info("%s - %s", existing.name, existing.timecode)
        self.dbcon.commit()
        cur.close()

    def load_offsets(self, offset, frame_count):
        cur = self.dbcon.cursor()
        frames = cur.execute("SELECT id, offset FROM main.frames WHERE offset >= ? ORDER BY id LIMIT ?", [offset, frame_count]).fetchall()
        result = [f[1] for f in frames]
        cur.close()
        return result

    def timestamp_for_offset(self, offset):
        return self.start_datetime.add(microseconds=(offset*1000))

    def _load_timestamps(self):
        with (self.path / "video-frame.timestamps").open() as fp:
            stamps = fp.read()
        cur = self.dbcon.cursor()
        for i, frame in enumerate(stamps.split('\n')[1:]):
            if frame != "":
                cur.execute('INSERT OR IGNORE INTO main.frames (id, offset) VALUES (?, ?)', [i, frame])
                # tc = Timecode.from_offset(float(frame), 10)
        self.dbcon.commit()
        cur.close()



import json
import subprocess

from cachetools.func import ttl_cache
import ffmpeg
import jmespath


FPS_PATH = jmespath.compile("streams[?codec_type=='video'].r_frame_rate")
FRC_PATH = jmespath.compile("streams[?codec_type=='video'].nb_read_frames")


def h264_to_mp4(input_name, output_name, frame_rate=10):
    (
        ffmpeg
        .input(input_name, format="h264", r=str(frame_rate))
        .output(output_name, **{"format": "mp4", "c:v": "copy", "r": str(frame_rate)})
        .run(quiet=False)
    )


@ttl_cache(ttl=60 * 60 * 4)
def get_video_file_details(path):
    ffprobe_out = subprocess.run([
        "ffprobe",
        "-v",
        "error",
        "-show_entries",
        "format=duration:stream=nb_read_frames,r_frame_rate,codec_type",
        "-count_frames",
        "-of",
        "json=compact=1",
        path,
    ], capture_output=True, check=True)
    probe_data = json.loads(ffprobe_out.stdout)
    fps = eval(FPS_PATH.search(probe_data)[0])  # pylint: disable=W0123
    frame_count = int(FRC_PATH.search(probe_data)[0])
    return {
        "fps": fps,
        "frame_count": frame_count,
        "duration":frame_count / fps
    }



from datetime import datetime
import os
import socket

import cdp
import logging


POSITION_V3 = 0x0135
ACCELEROMETER_V2 = 0x0139
GYROSCOPE_V2 = 0x013A
MAGNETOMETER_V2 = 0x013B
PRESSURE_V2 = 0x013C
QUATERNION_V2 = 0x013D
TEMPERATURE_V2 = 0x013E
DEVICE_NAMES = 0x013F
HARDWARE_STATUS_V2 = 0x0138
ANCHOR_HEALTH_V5 = 0x014A
NETWORK_TIME_MAPPING_V1 = 0x015A

MAX_TIMESTAMP_REFRESH_DELAY = 60  # seconds before we lose confidence in our interpolated real time value


DEBUG = os.getenv('DEBUG', "False").lower() in ('true', '1', 't')

class CUWBNetworkTimeMapping:
    def __init__(self,
                 last_real_time_previous,
                 last_real_time_current,
                 last_network_time_previous,
                 last_network_time_current):
        self.last_real_time_previous = last_real_time_previous
        self.last_real_time_current = last_real_time_current
        self.last_network_time_previous = last_network_time_previous
        self.last_network_time_current = last_network_time_current

    def rt_per_nt(self):
        return (self.last_real_time_current - self.last_real_time_previous) /\
               (self.last_network_time_current - self.last_network_time_previous)

    def interpolate_real_time_from_network_time(self, network_time):
        """
        Determine real time from network time

        :param network_time: As picoseconds
        :return: Real time as microseconds
        """
        return self.last_real_time_current +\
               ((network_time - self.last_network_time_current) * self.rt_per_nt())

    @staticmethod
    def real_time_as_date_time(real_time):
        return datetime.utcfromtimestamp(real_time/1e6)


class CUWBCollector:

    def __init__(self, ip, port, interface):
        self.ip = ip
        self.port = port
        self.interface = interface
        self.listen_socket = None

    def start(self):
        if not self.listen_socket:
            self.listen_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            self.listen_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            self.listen_socket.bind((self.ip, self.port))
            try:
                self.listen_socket.setsockopt(socket.SOL_IP, socket.IP_ADD_MEMBERSHIP, socket.inet_aton(self.ip) + socket.inet_aton(self.interface))
            except:
                logging.error("Failed connecting to socket through remote IP, falling back to a non-routed local connection")

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, type, value, traceback):
        self.listen_socket.close()
        self.listen_socket = None

    def extract_data_items(self, socket_read_time, data_item_type, type_name, cdp_packet, fields, last_time_mapping: CUWBNetworkTimeMapping=None, debug=False):
        for item in cdp_packet.data_items_by_type.get(data_item_type, []):
            if debug:
                logging.warning("Logging {}: Definition: {} - Full Item: {}".format(type_name, item.definition, item))

            timestamp = None
            if 'network_time' in fields:
                network_time = getattr(item, 'network_time')

                if last_time_mapping is None:
                    logging.warning("Skipping '{}': Data item contains 'network_time', but a CUWBNetworkTime has not yet been interpolated".format(type_name))
                    return

                # 4/27/21 - Stop excluding instances when last synced network time is a more recent timestamp
                #           than the timestamp in the IMU record
                # if network_time < last_time_mapping.last_network_time_current:
                #     logging.warning("Skipping '{}': Data item's 'network_time' is less than the last recorded CUWBNetworkTime. Network has likely reset. Waiting for CUWBNetworkTime to refresh.".format(type_name))
                #     return

                seconds_since_refresh = network_time_to_seconds(network_time - last_time_mapping.last_network_time_current)
                if seconds_since_refresh > MAX_TIMESTAMP_REFRESH_DELAY:
                    logging.warning("Skipping '{}': Data item contains 'network_time', but CUWBNetworkTime hasn't been refreshed for {} seconds. Waiting for CUWBNetworkTime refresh.".format(type_name, seconds_since_refresh))
                    return

                interpolated_real_time = last_time_mapping.interpolate_real_time_from_network_time(network_time)
                timestamp = CUWBNetworkTimeMapping.real_time_as_date_time(interpolated_real_time)

            data = {
                'socket_read_time': socket_read_time,
                'timestamp': timestamp,
                'type': type_name,
            }
            for field in fields:
                if hasattr(item, field):
                    data[field] = getattr(item, field)
                    if field == 'serial_number':
                        data[field] = str(data[field])
                    if field == 'bad_paired_anchors':
                        data[field] = ','.join([str(di) for di in data[field]])

            yield data

    def __iter__(self):
        self.start()

        last_time_mapping = None
        while True:
            try:
                data, address = self.listen_socket.recvfrom(65535)  # 2^16 is the maximum size of a CDP packet
                socket_read_time = datetime.utcnow()
                cdp_packet = cdp.CDP(data)
            except ValueError:
                logging.error("Failed parsing socket content")
                continue

            logging.info("Packet received at: {}".format(socket_read_time))

            fields = [
                'real_time_previous',
                'real_time_current',
                'network_time_previous',
                'network_time_current']
            for item in self.extract_data_items(socket_read_time, NETWORK_TIME_MAPPING_V1, 'network_time', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                # Resync network time
                last_time_mapping = CUWBNetworkTimeMapping(last_real_time_previous=item['real_time_previous'],
                                                           last_real_time_current=item['real_time_current'],
                                                           last_network_time_previous=item['network_time_previous'],
                                                           last_network_time_current=item['network_time_current'])
                yield item

            fields = [
                'serial_number',
                'network_time',
                'x',
                'y',
                'z',
                'scale',
            ]
            for item in self.extract_data_items(socket_read_time, ACCELEROMETER_V2, 'accelerometer', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item
            for item in self.extract_data_items(socket_read_time, GYROSCOPE_V2, 'gyroscope', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item
            for item in self.extract_data_items(socket_read_time, MAGNETOMETER_V2, 'magnetometer', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            fields = [
                'serial_number',
                'network_time',
                'x',
                'y',
                'z',
                'w',
                'quaternion_type',
            ]
            for item in self.extract_data_items(socket_read_time, QUATERNION_V2, 'quaternion', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            for item in self.extract_data_items(socket_read_time, PRESSURE_V2, 'pressure', cdp_packet, ['serial_number', 'network_time', 'pressure', 'scale'], last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

                for item in self.extract_data_items(socket_read_time, TEMPERATURE_V2, 'temperature', cdp_packet, ['serial_number', 'network_time', 'temperature', 'scale'], last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            for item in self.extract_data_items(socket_read_time, DEVICE_NAMES, 'names', cdp_packet, ['serial_number', 'name'], last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            fields = [
                'serial_number',
                'network_time',
                'x',
                'y',
                'z',
                'anchor_count',
                'quality',
                'flags',
                'smoothing',
            ]
            for item in self.extract_data_items(socket_read_time, POSITION_V3, 'position', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            fields = [
                'serial_number',
                'memory',
                'flags',
                'minutes_remaining',
                'battery_percentage',
                'temperature',
                'processor_usage',
            ]
            for item in self.extract_data_items(socket_read_time, HARDWARE_STATUS_V2, 'status', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item

            fields = [
                'serial_number',
                'interface_id',
                'ticks_reported',
                'timed_rxs_reported',
                'beacons_reported',
                'beacons_discarded',
                'beacons_late',
                'average_quality',
                'report_period',
                'interanchor_comms_error_code',
                'bad_paired_anchors',
            ]
            for item in self.extract_data_items(socket_read_time, ANCHOR_HEALTH_V5, 'anchor_health', cdp_packet, fields, last_time_mapping=last_time_mapping, debug=DEBUG):
                yield item


def network_time_to_seconds(ut):
    return float(ut) * 15.65 / (1e12)



import json


class DatabaseConnectionSnoop():
    def write_datapoint_object_time_series(
        self,
        object_id,
        data
    ):
        value_dict = {
            'object_id': object_id
        }
        value_dict.update(data)
        value_dict['timestamp'] = value_dict['timestamp'].isoformat() if value_dict.get('timestamp', None) else None
        value_dict['socket_read_time'] = value_dict['socket_read_time'].isoformat() if value_dict.get('socket_read_time', None) else None
        print(json.dumps(value_dict))



import logging
import click

from capture.segmentparser import monitor_directory


logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s - %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')


@click.group()
def main():
    pass

#
# @main.command()
# def capture():
# from capture.legacy import capture_loop
#     capture_loop()
#
#
# @main.command()
# def upload():
# from capture.legacy import upload_loop
#     upload_loop()


@main.command()
@click.argument('path')
def upload_monitor(path):
    monitor_directory(path)


if __name__ == '__main__':
    main()



import os
import socket

import click

from cuwb_stream.collector import CUWBCollector
from cuwb_stream.network import ensure_network_is_running, ensure_network_is_stopped
from cuwb_stream.tools.snooplogg import DatabaseConnectionSnoop


def get_local_ip(routable_ip='8.8.8.8'):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect((routable_ip, 80))
    ip = s.getsockname()[0]
    s.close()
    return ip


@click.group()
@click.pass_context
def main(ctx):
    pass


@main.command()
@click.pass_context
@click.option('--consumer', help='the consumer type, supports `stdout`, `csv`, or `honeycomb` at this time')
@click.option('--environment_name_honeycomb', help='name of the environment in honeycomb, required for using the honeycomb consumer')
@click.option('--path_csv', help='path for csv file output, required for using the csv consumer')
@click.option('--ip', help='Socket ip for the CUWB network. (defaults to CUWB_SOCKET_IP env or 239.255.76.67)')
@click.option('--port', help='Socket port for the CUWB network. (defaults to CUWB_SOCKET_PORT env or 7667)')
@click.option('--route_ip', help='IP that the interface should route to. (defaults to CUWB_SOCKET_PORT env or 8.8.8.8)')
def collect(ctx, consumer, environment_name_honeycomb=None, path_csv=None, ip=None, port=None, route_ip=None):
    if consumer not in ['stdout']:
        raise ValueError('consumer must be `stdout`')

    database_connection = DatabaseConnectionSnoop()

    ip = ip if ip else os.environ.get('CUWB_SOCKET_IP', '0.0.0.0')
    port = port if port else os.environ.get('CUWB_SOCKET_PORT', 7667)
    interface = route_ip if route_ip else get_local_ip(os.environ.get('CUWB_ROUTABLE_IP', '8.8.8.8'))
    with CUWBCollector(ip, int(port), interface) as collector:
        for bit in collector:
            if bit:
                database_connection.write_datapoint_object_time_series(
                    object_id=bit.get("serial_number"),
                    data=bit
                )


@main.command()
@click.pass_context
@click.option('--name', help='name of the network')
@click.option('--action', help='start or stop')
def network(ctx, name, action):
    if action == "start":
        ensure_network_is_running(name)
    elif action == "stop":
        ensure_network_is_stopped(name)


@main.command()
@click.pass_context
@click.option('--file', help='path to the file to upload')
@click.option('--serial_number', help='serial_number for the device')
@click.option('--start', help='starting timestamp')
@click.option('--type', help='type of measurement(s)')
@click.option('--environment_name_honeycomb', help='name of the environment in honeycomb, required for using the honeycomb consumer')
def upload(ctx, environment_name_honeycomb, file, serial_number, start, type):
    from database_connection_honeycomb import DatabaseConnectionHoneycomb
    database_connection = DatabaseConnectionHoneycomb(
        environment_name_honeycomb=environment_name_honeycomb,
        object_type_honeycomb="DEVICE",
        object_id_field_name_honeycomb="serial_number"
    )
    with open(file, 'r') as fp:
        database_connection.write_datapoint_object_time_series(
            timestamp=start,
            object_id=serial_number,
            data=fp.read()
        )


if __name__ == '__main__':
    main()
